/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"
#uses "fwSynopticLocator/fwSynopticLocator.ctl"

const string FWSYPNOPTIC_DP_NAME  = "_fwSynopticLocator_System_Communication";
dyn_string dsPaths, dsDevices, dsPanelTitles;

//-------------------------------------------------------
void test_fwSynopticLocator_setupSuite() {
	if (dpExists(FWSYPNOPTIC_DP_NAME)) {

      dpGet(COMMUNICATION_DPE_PATHS, dsPaths,
            COMMUNICATION_DPE_DEVICE_INDENTIFIER, dsDevices,
            COMMUNICATION_DPE_PANEL_TITLE_IDENTIFIER, dsPanelTitles);
	
      dyn_string dsProjPaths;
      fwInstallation_getProjPaths(dsProjPaths);

      for (int i=1; i<= dynlen(dsProjPaths); i++) {
        dsProjPaths[i] = dsProjPaths[i] + "/panels/vision/test/fwSynopticLocator/build";
      }

      string device = "sIdentifier";
      string panelTitle = "sNone";

      dpSetWait(COMMUNICATION_DPE_PATHS, dsProjPaths);
      dpSetWait(COMMUNICATION_DPE_DEVICE_INDENTIFIER, device);
      dpSetWait(COMMUNICATION_DPE_PANEL_TITLE_IDENTIFIER, panelTitle);

	} else {
      DebugTN("WARNING! test_fwSynopticLocator_setupSuite skipped. FwSynoptic Configuration _fwSynopticLocator_System_Communication datapoint does not exist.");
	}
}

//-------------------------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_fwSynopticLocator_alwaysRunInUI()
{
}

//---------------------------------------------------------
void simulateFwSynopticLocator_Clean() {
	
	if (dpExists(FWSYPNOPTIC_DP_NAME)) {

	  dyn_string dsRes;
	  dyn_float dfRes;

	  ChildPanelOnReturn("vision/test/fwSynopticLocator/test_fwSynopticLocator_Clean.xml",
								  "testCleanSynoptic",
								  makeDynString(""),
								  -1, -1,
								  dfRes,
								  dsRes);
	  delay(2);

	  dyn_bool expectedDpPanelsExisting = makeDynBool(false, false);
	  dyn_bool expectedDpDevicesExisting = makeDynBool(false, false, false, false, false, false);
	
	  dyn_bool actualDpPanelsExisting = makeDynBool(dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel1_PNL"),
                                                    dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel2_PNL"));

	  dyn_bool actualDpDevicesExisting = makeDynBool(dpExists("_fwSynopticLocator_Device_Test1"), dpExists("_fwSynopticLocator_Device_Test2"),
                                                     dpExists("_fwSynopticLocator_Device_Test3"), dpExists("_fwSynopticLocator_Device_Test4"),
                                                     dpExists("_fwSynopticLocator_Device_Test5"), dpExists("_fwSynopticLocator_Device_Test6"));
	
	  assertEqual(expectedDpPanelsExisting, actualDpPanelsExisting, "Clean failed. Synoptic locator panels exist when they should not");
	  assertEqual(expectedDpDevicesExisting, actualDpDevicesExisting, "Clean failed. Synoptic locator devices exist when they should not");
	} else {
	  DebugTN("WARNING! test_fwSynopticLocator_Clean skipped. FwSynoptic Configuration _fwSynopticLocator_System_Communication datapoint does not exist.");
	}
}


//------------------------------------------------------
void simulateFwSynopticLocator_Build() {

	if (dpExists(FWSYPNOPTIC_DP_NAME)) {

	  dyn_string dsRes;
	  dyn_float dfRes;

	  ChildPanelOnReturn("vision/test/fwSynopticLocator/test_fwSynopticLocator_Build.xml",
								  "testBuildSynoptic",
								  makeDynString(""),
								  -1, -1,
								  dfRes,
								  dsRes);
	  delay(2);

	  dyn_bool expectedDpPanelsExisting = makeDynBool(true, true);
	  dyn_bool expectedDpDevicesExisting = makeDynBool(true, true, true, true, true, true);
	
	  dyn_bool actualDpPanelsExisting = makeDynBool(dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel1_PNL"),
                                                    dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel2_PNL"));

	  dyn_bool actualDpDevicesExisting = makeDynBool(dpExists("_fwSynopticLocator_Device_Test1"), dpExists("_fwSynopticLocator_Device_Test2"),
                                                     dpExists("_fwSynopticLocator_Device_Test3"), dpExists("_fwSynopticLocator_Device_Test4"),
                                                     dpExists("_fwSynopticLocator_Device_Test5"), dpExists("_fwSynopticLocator_Device_Test6"));
	
	  assertEqual(expectedDpPanelsExisting, actualDpPanelsExisting, "Build failed. Synoptic locator panels do not exist when they should");
	  assertEqual(expectedDpDevicesExisting, actualDpDevicesExisting, "Build failed. Synoptic locator devices do not exist when they should");
	} else {
	  DebugTN("WARNING! test_fwSynopticLocator_Build skipped. FwSynoptic Configuration _fwSynopticLocator_System_Communication datapoint does not exist.");
	}
}

//---------------------------------------------------------
void simulateFwSynopticLocator_Update() {

	if (dpExists(FWSYPNOPTIC_DP_NAME)) {
	  dyn_string dsRes;
	  dyn_float dfRes;
	
	  dyn_string dsProjPaths;
       fwInstallation_getProjPaths(dsProjPaths);

       for (int i=1; i<= dynlen(dsProjPaths); i++) {
         dsProjPaths[i] = dsProjPaths[i] + "/panels/vision/test/fwSynopticLocator/update";
       }

	  dpSetWait(COMMUNICATION_DPE_PATHS, dsProjPaths);

	  ChildPanelOnReturn("vision/test/fwSynopticLocator/test_fwSynopticLocator_Update.xml",
								  "testUpdateSynoptic",
								  makeDynString(""),
								  -1, -1,
								  dfRes,
								  dsRes);
	  delay(2);
	
	  dyn_bool expectedDpPanelsExisting = makeDynBool(false, false, true);
	  dyn_bool expectedDpDevicesExisting = makeDynBool(true, true, true, true, true, true, true, true, true);
	
	  dyn_bool actualDpPanelsExisting = makeDynBool(dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel1_PNL"),
                                                    dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel2_PNL"),
                                                    dpExists("_fwSynopticLocator_Panel_test_fwSynopticLocator_Panel3_PNL"));

	  dyn_bool actualDpDevicesExisting = makeDynBool(dpExists("_fwSynopticLocator_Device_Test1"), dpExists("_fwSynopticLocator_Device_Test2"),
                                                     dpExists("_fwSynopticLocator_Device_Test3"), dpExists("_fwSynopticLocator_Device_Test3"),
                                                     dpExists("_fwSynopticLocator_Device_Test5"), dpExists("_fwSynopticLocator_Device_Test6"),
                                                     dpExists("_fwSynopticLocator_Device_Test7"), dpExists("_fwSynopticLocator_Device_Test8"),
                                                     dpExists("_fwSynopticLocator_Device_Test9"));
	
	  assertEqual(expectedDpPanelsExisting, actualDpPanelsExisting, "Update failed. Existance of synoptic locator panels do not match with what is expected");
	  assertEqual(expectedDpDevicesExisting, actualDpDevicesExisting, "Update failed. Existance of synoptic locator devices do not match with what is expected");
	} else {
	  DebugTN("WARNING! test_fwSynopticLocator_Update skipped. FwSynoptic Configuration _fwSynopticLocator_System_Communication datapoint does not exist.");
	}
}

//---------------------------------------------------------
// Tests the sequence Clean --> Build --> Clean
// The first clean is done in case that a previous fwSynopticLocator configuration remains.
// Build is executed afterwards to setup a configuration. To finalize, clean is executed 
// again to check that the configuration created by build is removed.

void test_fwSynopticLocator_BuildCleanSequence() {
	simulateFwSynopticLocator_Clean();
	simulateFwSynopticLocator_Build();
	simulateFwSynopticLocator_Clean();
}

//---------------------------------------------------------
// Tests the sequence Clean --> Build --> Update --> Clean
// The first clean is done in case that a previous fwSynopticLocator configuration remains.
// Build is executed afterwards to setup a configuration. Update is called after build to
// check that the previous configuration changes. To finalize, clean is executed again to 
// check that the configuration created by build and modified by update is removed.

void test_fwSynopticLocator_BuildUpdateCleanSequence() {
	simulateFwSynopticLocator_Clean();
	simulateFwSynopticLocator_Build();
	simulateFwSynopticLocator_Update();
	simulateFwSynopticLocator_Clean();
}

//---------------------------------------------------------
void test_fwSynopticLocator_teardownSuite() {
	if (dpExists(FWSYPNOPTIC_DP_NAME)) {
	  dpSetWait(COMMUNICATION_DPE_PATHS, dsPaths);
	  dpSetWait(COMMUNICATION_DPE_DEVICE_INDENTIFIER, dsDevices);
	  dpSetWait(COMMUNICATION_DPE_PANEL_TITLE_IDENTIFIER, dsPanelTitles);
	} else {
	  DebugTN("WARNING! test_fwSynopticLocator_teardownSuite skipped. FwSynoptic Configuration _fwSynopticLocator_System_Communication datapoint does not exist.");
	}
}
